"""Text Utilities."""

# flake8: noqa


from __future__ import annotations

import re
from difflib import SequenceMatcher
from typing import Iterable, Iterator

from kombu import version_info_t


def escape_regex(p, white=''):
    # type: (str, str) -> str
    """Escape string for use within a regular expression."""
    # what's up with re.escape? that code must be neglected or something
    return ''.join(
        c if c.isalnum() or c in white else ('\\000' if c == '\000' else '\\' + c)
        for c in p
    )


def fmatch_iter(
    needle: str, haystack: Iterable[str], min_ratio: float = 0.6
) -> Iterator[tuple[float, str]]:
    """Fuzzy match: iteratively.

    Yields
    ------
        Tuple: of ratio and key.
    """
    for key in haystack:
        ratio = SequenceMatcher(None, needle, key).ratio()
        if ratio >= min_ratio:
            yield ratio, key


def fmatch_best(
    needle: str, haystack: Iterable[str], min_ratio: float = 0.6
) -> str | None:
    """Fuzzy match - Find best match (scalar)."""
    try:
        return sorted(
            fmatch_iter(needle, haystack, min_ratio),
            reverse=True,
        )[
            0
        ][1]
    except IndexError:
        return None


def version_string_as_tuple(version: str) -> version_info_t:
    """Parse a version string into its components and return a version_info_t tuple.

    The version string is expected to follow the pattern:
    'major.minor.micro[releaselevel][serial]'. Each component of the version
    is extracted and returned as a tuple in the format (major, minor, micro,
    releaselevel, serial).

    Args
    ----
        version (str): The version string to parse.

    Returns
    -------
        version_info_t: A tuple containing the parsed version components.

    Raises
    ------
        ValueError: If the version string is invalid and does not match the expected pattern.
    """
    pattern = r'^(\d+)'  # catching the major version (mandatory)
    pattern += r'(?:\.(\d+))?'  # optionally catching the minor version
    pattern += r'(?:\.(\d+))?'  # optionally catching the micro version
    pattern += r'(?:\.*([a-zA-Z+-][\da-zA-Z+-]*))?'  # optionally catching the release level (starting with a letter, + or -) after a dot
    pattern += r'(?:\.(.*))?'  # optionally catching the serial number after a dot

    # applying the regex pattern to the input version string
    match = re.match(pattern, version)

    if not match:
        raise ValueError(f"Invalid version string: {version}")

    # extracting the matched groups
    major = int(match.group(1))
    minor = int(match.group(2)) if match.group(2) else 0
    micro = int(match.group(3)) if match.group(3) else 0
    releaselevel = match.group(4) if match.group(4) else ''
    serial = match.group(5) if match.group(5) else ''

    return _unpack_version(major, minor, micro, releaselevel, serial)


def _unpack_version(
    major: str | int = 0,
    minor: str | int = 0,
    micro: str | int = 0,
    releaselevel: str = '',
    serial: str = '',
) -> version_info_t:
    return version_info_t(int(major), int(minor), int(micro), releaselevel, serial)


def _splitmicro(
    micro: str, releaselevel: str = '', serial: str = ''
) -> tuple[int, str, str]:
    for index, char in enumerate(micro):
        if not char.isdigit():
            break
    else:
        return int(micro or 0), releaselevel, serial
    return int(micro[:index]), micro[index:], serial



from __future__ import annotations

import pytest

from kombu import version_info_t
from kombu.utils.text import version_string_as_tuple


def test_dir():
    import kombu

    assert dir(kombu)


@pytest.mark.parametrize(
    'version,expected',
    [
        ('3', version_info_t(3, 0, 0, '', '')),
        ('3.3', version_info_t(3, 3, 0, '', '')),
        ('3.3.1', version_info_t(3, 3, 1, '', '')),
        ('3.3.1a3', version_info_t(3, 3, 1, 'a3', '')),
        ('3.3.1.a3.40c32', version_info_t(3, 3, 1, 'a3', '40c32')),
        ('4.0.0+beta.3.47.g4f1a05b', version_info_t(
            4, 0, 0, '+beta', '3.47.g4f1a05b')),
        ('4.0.0-beta3.47.g4f1a05b', version_info_t(
            4, 0, 0, '-beta3', '47.g4f1a05b')),
        ('4.0.1-alpha.3+40c32', version_info_t(4, 0, 1, '-alpha', '3+40c32')),
        ('0+beta3.14159265', version_info_t(0, 0, 0, '+beta3', '14159265')),
    ],
)
def test_version_string_as_tuple(version, expected):
    assert version_string_as_tuple(version) == expected



#!/usr/bin/env python3
from __future__ import annotations

import os
import re
import sys

import setuptools
from setuptools import setup

# -- Parse meta
re_meta = re.compile(r'__(\w+?)__\s*=\s*(.*)')
re_doc = re.compile(r'^"""(.+?)"""')


def add_default(m):
    attr_name, attr_value = m.groups()
    return ((attr_name, attr_value.strip("\"'")),)


def add_doc(m):
    return (('doc', m.groups()[0]),)


pats = {re_meta: add_default, re_doc: add_doc}
here = os.path.abspath(os.path.dirname(__file__))
meta_fh = open(os.path.join(here, 'kombu/__init__.py'))
try:
    meta = {}
    for line in meta_fh:
        if line.strip() == '# -eof meta-':
            break
        for pattern, handler in pats.items():
            m = pattern.match(line.strip())
            if m:
                meta.update(handler(m))
finally:
    meta_fh.close()


# --


def fullsplit(path, result=None):
    if result is None:
        result = []
    head, tail = os.path.split(path)
    if head == '':
        return [tail] + result
    if head == path:
        return result
    return fullsplit(head, [tail] + result)


# if os.path.exists('README.rst'):
#    long_description = codecs.open('README.rst', 'r', 'utf-8').read()
# else:
#    long_description = 'See https://pypi.org/project/kombu/'

# -*- Installation Requires -*-
py_version = sys.version_info
is_pypy = hasattr(sys, 'pypy_version_info')


def strip_comments(line):
    return line.split('#', 1)[0].strip()


def reqs(*f):
    with open(os.path.join(os.getcwd(), "requirements", *f)) as reqs_file:
        return [r for r in (strip_comments(line) for line in reqs_file) if r]


def extras(*p):
    return reqs('extras', *p)


def readme():
    with open('README.rst') as f:
        return f.read()


setup(
    name='kombu',
    packages=setuptools.find_packages(exclude=['t', 't.*']),
    version=meta['version'],
    description=meta['doc'],
    keywords='messaging message amqp rabbitmq redis actor producer consumer',
    author=meta['author'],
    author_email=meta['contact'],
    url=meta['homepage'],
    project_urls={
        'Source': 'https://github.com/celery/kombu'
    },
    platforms=['any'],
    license='BSD-3-Clause',
    python_requires=">=3.8",
    install_requires=reqs('default.txt'),
    tests_require=reqs('test.txt'),
    extras_require={
        'msgpack': extras('msgpack.txt'),
        'yaml': extras('yaml.txt'),
        'redis': extras('redis.txt'),
        'mongodb': extras('mongodb.txt'),
        'sqs': extras('sqs.txt'),
        'zookeeper': extras('zookeeper.txt'),
        'sqlalchemy': extras('sqlalchemy.txt'),
        'librabbitmq': extras('librabbitmq.txt'),
        'pyro': extras('pyro.txt'),
        'slmq': extras('slmq.txt'),
        'azurestoragequeues': extras('azurestoragequeues.txt'),
        'azureservicebus': extras('azureservicebus.txt'),
        'qpid': extras('qpid.txt'),
        'consul': extras('consul.txt'),
        'confluentkafka': extras('confluentkafka.txt'),
    },
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3 :: Only',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'Programming Language :: Python :: 3.11',
        'Programming Language :: Python :: 3.12',
        'Programming Language :: Python :: Implementation :: CPython',
        'Programming Language :: Python :: Implementation :: PyPy',
        'Intended Audience :: Developers',
        'Topic :: Communications',
        'Topic :: System :: Distributed Computing',
        'Topic :: System :: Networking',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],
)
